<?php
/* Copyright (c) 1998-2016 ILIAS open source, Extended GPL, see docs/LICENSE */

include_once("./Services/Export/classes/class.ilXmlImporter.php");
include_once("./Modules/TestQuestionPool/classes/class.ilObjQuestionPool.php");

/**
 * Class ilQuestionSetPoolImporter
 * @author Nadia Matuschek <nmatuschek@databay.de>
 */
class ilQuestionSetPoolImporter extends ilXmlImporter
{
	const SINGLE_IMPORT_NEW_POOL = -1;
	const SINGLE_IMPORT_NO_POOL  = -2;

	/**
	 * @var ilObjQuestionPool
	 */
	protected $qpl_obj = NULL;
	
	protected $is_single_import = true;
	
	/**
	 * @var int[]
	 */
	protected static $pool_by_parent_id = array();
	
	protected $current_parent_ref_id = 0;
	
	/**
	 * @var null
	 */
	protected $xml_file = NULL;
	/**
	 * @var null
	 */
	protected $qti_file = NULL;
	
	/**
	 * @var array
	 */
	protected $qpl_qst_mapping = array();
	
	/**
	 * @param $xml_file
	 */
	private function setXmlFile($xml_file)
	{
		$this->xml_file = $xml_file;
	}
	
	public function getXmlFile()
	{
		return $this->xml_file;
	}
	
	/**
	 * @param $qti_file
	 */
	public function setQtiFile($qti_file)
	{
		$this->qti_file = $qti_file;
	}
	
	public function getQtiFile()
	{
		return $this->qti_file;
	}
	
	/**
	 * ilQuestionSetPoolImporter constructor.
	 */
	public function __construct()
	{
		$this->plugin_object = ilQuestionSetPoolPlugin::getInstance();
		$this->plugin_object->includeClass('models/class.ilQuestionSetPoolSet.php');
		$this->plugin_object->includeClass('models/class.ilQuestionSetPoolPath.php');
		$this->plugin_object->includeClass('models/class.ilQuestionSetPoolNode.php');
		$this->plugin_object->includeClass('models/class.ilQuestionSetPoolJumpCondition.php');
	}
	
	public function init()
	{
		$qti_path = $this->getImportDirectory().'/Plugins/xqsp/set_1/expDir_1';
		$qti_file = $qti_path.'/xqsp_qti.xml';
		$xml_file = $this->getImportDirectory().'/Plugins/xqsp/set_1/export.xml';

		$this->validateXmlFile($xml_file);
		$this->setQtiFile($qti_file);
	}
	
	
	/**
	 * @param $xml_file
	 */
	public function validateXmlFile($xml_file)
	{
		$this->plugin_object->includeClass('class.ilQuestionSetPoolXmlValidator.php');
		$validator = new ilQuestionSetPoolXmlValidator();
		$validator->setXsdFile('/xml/xqsp_5_1.xsd');
		$validator->setXmlFile($xml_file);
		$validator->validate();
		$this->setXmlFile($xml_file);
		return true;
	}
	
	/**
	 * 
	 */
	public function createImportDirectory()
	{
		global $ilErr;
		
		include_once "./Services/Utilities/classes/class.ilUtil.php";
		$xqsp_data_dir = ilUtil::getDataDir()."/xqsp_data";
		ilUtil::makeDir($xqsp_data_dir);
		
		if(!is_writable($xqsp_data_dir))
		{
			$ilErr->raiseError("QuestionSetPool Data Directory (".$xqsp_data_dir.") not writeable.", $ilErr->error_obj->FATAL);
		}
		
		// create QuestionSetPool directory (data_dir/xqsp_data/xqsp_import)
		$import_dir = $xqsp_data_dir."/xqsp_import";
		ilUtil::makeDir($import_dir);
		if(!@is_dir($import_dir))
		{
			$ilErr->raiseError("Creation of QuestionSetPool import directory failed.", $ilErr->error_obj->FATAL);
		}
		
		$this->setImportDirectory($import_dir);
	}
	
	public function initializeQplObject($id)
	{
		if($id <= 0)
		{
			$this->qpl_obj = $this->createQplObject();
		}	
		else
		{
			$this->qpl_obj = new ilObjQuestionPool($id, false);
			$this->qpl_obj->read(true);
		}
	}	
	
	/**
	 * Creates a new questionpool object
	 * 
	 * copied from ilObjQuestionPoolGUI->importVerifiedFileObject()
	 */
	public function createQplObject()
	{
		// create new questionpool object
		$qpl_obj = new ilObjQuestionPool(0, true);
		// set type of questionpool object
		$qpl_obj->setType('qpl');
		// set title of questionpool object to "dummy"
		$qpl_obj->setTitle("dummy");
		// set description of questionpool object
		$qpl_obj->setDescription("questionpool creation");
		// set online
		$qpl_obj->setOnline(true);
		// create the questionpool class in the ILIAS database (object_data table)
		$qpl_obj->create(true);
		// create a reference for the questionpool object in the ILIAS database (object_reference table)
		$qpl_obj->createReference();
		
		$qpl_obj->update();
		$qpl_obj->saveToDb();
		return $qpl_obj;
	}
	
	/**
	 * Import questions to QPL questionpool
	 */
	protected function parseQtiFile()
	{
		include_once "./Modules/TestQuestionPool/classes/class.ilObjQuestionPool.php";
		$tmp_dir = ilObjQuestionPool::_getImportDirectory();
		ilObjQuestionPool::_setImportDirectory($this->getImportDirectoryBase());

		$tmp_subdir = $_SESSION["qpl_import_subdir"];
		$_SESSION["qpl_import_subdir"] = basename($this->getImportDirectory());

		// start parsing of QTI files
		include_once "./Services/QTI/classes/class.ilQTIParser.php";
		$qtiParser = new ilQTIParser($this->getQtiFile(), IL_MO_PARSE_QTI, $this->xqsp_obj->getId());
		$result = $qtiParser->startParsing();
		$this->addQuestionMapping($qtiParser->getImportMapping());

		ilObjQuestionPool::_setImportDirectory($tmp_dir);
		$_SESSION["qpl_import_subdir"] = $tmp_subdir;
	}
	
	protected function addQuestionMapping($qtiParser_import_mapping)
	{
		foreach($qtiParser_import_mapping as $question_ident => $pool_qst_ident)
		{
			$exp_ident = explode('_qst_', $question_ident);
			$old_qst_id = end($exp_ident);
			$new_qst_id = $pool_qst_ident['pool'];
			$this->qpl_qst_mapping[$old_qst_id] = $new_qst_id; 
		}
	}

	/**
	 * @param array $set_data
	 * @param array $set_questions
	 * @param array $set_paths
	 */
	public function importQuestionSet($set_data, $set_questions, $set_paths, $a_mapping)
	{
		$set = new ilQuestionSetPoolSet();
		$set->setContainerObjId($this->xqsp_obj->getId());
		$set->setTitle($set_data['SetTitle']);
		$set->setDescription($set_data['SetDescription']);
		$set->setOriginalId($set_data['OriginialId']);

		$set->update();

		$this->addSetMapping($set_data['SetId'], $set->getId());

		foreach($set_questions as $set_question)
		{
			$original_question_id = $a_mapping->getMapping('Modules/TestQuestionPool', 'quest', $set_question['QuestionOriginalId']);
			if($original_question_id)
			{
				if(!assQuestion::_questionExistsInPool($original_question_id))
				{
					$original_question_id = NULL;
				}
			}

			$copy_id = $set->getQuestionList()->import($this->qpl_qst_mapping[$set_question['QuestionId']], $original_question_id);
			$this->qpl_qst_mapping[$set_question['QuestionId']] = $copy_id;

			if($original_question_id == 0 || $this->is_single_import)
			{
				$this->qpl_qst_mapping['new_qpl'][$set_question['QuestionId']] = $copy_id;
				if($set_question['QuestionOriginalId'])
				{
					$a_mapping->addMapping('Plugins/xqsp', 'org_qst_id', $set_question['QuestionOriginalId'], $copy_id);
				}
			}
			else
			{
				$this->qpl_qst_mapping['new_qpl'][$set_question['QuestionId']] = $original_question_id;
			}
		}

		foreach($set_paths as $set_path)
		{
			$path = new ilQuestionSetPoolPath();
			$path->setSet($set);
			$path->setTitle($set_path[0]['PathTitle']);
			$path->import();

			$this->addPathMapping($set_data[0]['PathId'], $path->getId());

			$nodes = $set_path['nodes'];
			$this->importNodes($nodes, $path);
		}
	}
	
	/**
	 * @param array                 $nodes
	 * @param ilQuestionSetPoolPath $path
	 */
	protected function importNodes($nodes, ilQuestionSetPoolPath $path)
	{
		foreach($nodes as $node_data)
		{
			$node = new ilQuestionSetPoolNode();
			$node->setQuestion(assQuestion::_instantiateQuestion($this->qpl_qst_mapping[$node_data[0]['NodeQuestionId']]));
			$node->setPath($path);
			$node->setInitialCondition($node_data[0]['NodeInitialCondition']);
			if(strlen($node_data[0]['NodeDominantScoring']))
			{
				$node->setDominantScoring($node_data[0]['NodeDominantScoring']);
			}
			$node->setTrueFeedback($node_data[0]['NodeTrueFeedback']);
			$node->setFalseFeedback($node_data[0]['NodeFalseFeedback']);
			
			$node->import();
			
			$this->addNodeMapping($node_data[0]['NodeId'], $node->getId());
			
			$jump_conditions = $node_data['jump_conditions'];
			if(count($jump_conditions) == 0)
			{
				continue 1;
			}
			$this->importJumpConditions($jump_conditions, $node);
		}
	}
	
	/**
	 * @param array                 $jump_conditions
	 * @param ilQuestionSetPoolNode $node
	 */
	protected function importJumpConditions($jump_conditions, ilQuestionSetPoolNode $node)
	{
		foreach($jump_conditions as $jump_condition_data)
		{
			$jump_con = new ilQuestionSetPoolJumpCondition();
			$jump_con->setNode($node);
			$jump_con->setQuestion(
				$this->qpl_qst_mapping[$jump_condition_data['JumpConditionQuestionId']] ?
				assQuestion::_instantiateQuestion($this->qpl_qst_mapping[$jump_condition_data['JumpConditionQuestionId']]):
				NULL
			);
			$jump_con->setJumpCondition($jump_condition_data['JumpConditionText']);
			if(strlen($jump_condition_data['JumpConditionRepetitions']))
			{
				$jump_con->setRepititions($jump_condition_data['JumpConditionRepetitions']);
			}

			$jump_con->setTrueFeedback($jump_condition_data['JumpConditionTrueFeedback']);
			$jump_con->setTrueJumpFeedback($jump_condition_data['JumpConditionTrueJumpFeedback']);
			$jump_con->setFalseFeedback($jump_condition_data['JumpConditionFalseFeedback']);
			
			$jump_con->import();
		}
	}
	
	/**
	 * @param $old_set_id
	 * @param $new_set_id
	 */
	protected function addSetMapping($old_set_id, $new_set_id)
	{
		$this->set_mapping[$old_set_id] = $new_set_id;
	}
	
	/**
	 * @param $old_path_id
	 * @param $new_path_id
	 */
	protected function addPathMapping($old_path_id, $new_path_id)
	{
		$this->path_mapping[$old_path_id] = $new_path_id;
	}
	
	/**
	 * @param $old_node_id
	 * @param $new_node_id
	 */
	protected function addNodeMapping($old_node_id, $new_node_id)
	{
		$this->node_mapping[$old_node_id] = $new_node_id;
	}
	
	/**
	 * Import XML
	 * @param $a_entity
	 * @param $a_id
	 * @param $a_xml
	 * @param $a_mapping
	 * @return bool
	 */
	function importXmlRepresentation($a_entity, $a_id, $a_xml, $a_mapping)
	{
		/**
		 * @var $tree ilTree
		 */
		global $tree;

		$this->plugin_object->includeClass('class.ilObjQuestionSetPool.php');
		$this->plugin_object->includeClass('class.ilObjQuestionSetPoolXMLParser.php');
		
		$parent_ref_id = 0;
		
		if($new_id = $a_mapping->getMapping('Services/Container', 'objs', $a_id))
		{
			$ref_ids = ilObject::_getAllReferences($new_id);
			$ref_id  = current($ref_ids);
			
			$parent_ref_id = $tree->getParentId($ref_id);
			
			$this->xqsp_obj = ilObjectFactory::getInstanceByObjId($new_id, false);
			$this->xqsp_obj->setRefId($ref_id);
			$this->is_single_import = false;
		}
		else
		{
			$this->xqsp_obj = new ilObjQuestionSetPool();
			$this->xqsp_obj->create(true);
			$this->is_single_import = true;
		}
		
		$this->current_parent_ref_id = $parent_ref_id;
		
		// import questions to QPL questionpool		
		$this->parseQtiFile();

		$parser = new ilObjQuestionSetPoolXMLParser($this->xqsp_obj, $this->getXmlFile());
		$parser->setImportDirectory($this->getImportDirectory());
		$parser->startParsing();
		$this->xqsp_obj = $parser->getQuestionSetPoolObject();
		$this->xqsp_obj->update();
		
		// create sets
		$sets = $parser->getSetsArray();
		
		foreach($sets as $set_data)
		{
			$this->importQuestionSet($set_data[0], $set_data['questions'],  $set_data['paths'], $a_mapping);
		}
		
		$a_mapping->addMapping('Plugins/xqsp', 'xqsp', $a_id, $this->xqsp_obj->getId());
	}
	
	function finalProcessing($a_mapping)
	{
		
			// Container import
			// Prüfen, ob in einer statischen Variable (z.B. self::$pool_by_parent[$this->current_parent_ref_id] schon eine Pool ID existiert
			// Wenn ja, diesen Pool nehmen (sofern Bedarf für neue Originale)
			// Wenn nein, neuen Pool auf dieser Ebene -> mit putInTree($this->current_parent_ref_id) einhängen,
			// und dann dessen  ID hier merken: self::$pool_by_parent[$this->current_parent_ref_id]
		
		// Einzel-Import mit (ggf. erfolgter Wahl eines bestehenden Fragenpools als Ziel für Originale unserer Fragen)
		$new_qpl_id = (int)$_POST['qpl'];
		if($_POST['qpl'] == self::SINGLE_IMPORT_NO_POOL)
		{
			//Die neuen Fragen 'ohne pool' liegen schon im XQSP.
		}
		else
		{
			if ($new_qpl_id > 0)
			{
				$this->initializeQplObject($new_qpl_id);
			}
			// Containter-Import (Plugin liegt im Kurs, die Originale unserer Fragen liegen ggf.(!!) in ebenfalls importierten Fragenpools)
			else if ($this->is_single_import || count($this->qpl_qst_mapping['new_qpl']) > 0)
			{
				$parent_ref_id = $a_mapping->getTargetId();
				if (!$this->is_single_import && $this->current_parent_ref_id > 0)
				{
					$parent_ref_id = $this->current_parent_ref_id;
				}

				if (isset(self::$pool_by_parent_id[$parent_ref_id]))
				{
					$pool_obj_id = self::$pool_by_parent_id[$parent_ref_id];
					$this->initializeQplObject($pool_obj_id);
				}
				else
				{
					// es handelt sich um einen dummy pool
					$this->initializeQplObject(0);
					// put the questionpool object in the administration tree
					$this->qpl_obj->putInTree($parent_ref_id);
					// get default permissions and set the permissions for the questionpool object
					$this->qpl_obj->setPermissions($parent_ref_id);

					self::$pool_by_parent_id[$parent_ref_id] = $this->qpl_obj->getId();
				}
			}

			foreach ($this->qpl_qst_mapping['new_qpl'] as $new_id)
			{
				//Die neuen Fragen 'ohne pool' liegen schon im XQSP.
				// jetzt müssen sie in den neu angelegten dummy pool (QPL) kopiert werden und die org_id im xqsp-pool angepasst werden.
				$source_qst   = assQuestion::_instantiateQuestion($new_id);
				$is_xqsp_pool = (ilObject::_lookupType($source_qst->getObjId()) == 'xqsp' ? true : false);
				// This is the intended behaviour, see http://www.ilias.de/mantis/view.php?id=18325
				if(($source_qst->getOriginalId() == NULL || $this->is_single_import) && $is_xqsp_pool)
				{
					// Dies ist nur für Fragen "ohne pool" relevant!!!!
					$clone_id   = $source_qst->copyObject($this->qpl_obj->getId());
					$source_qst = assQuestion::_instantiateQuestion($new_id);
					$source_qst->setNewOriginalId($clone_id);
					$source_qst->saveToDb();
				}
			}
		}
		return parent::finalProcessing($a_mapping);
	}
	
	/**
	 * Create qti and xml file name
	 * @return array 
	 */
	protected function parseXmlFileNames()
	{
		$GLOBALS['ilLog']->write(__METHOD__.': '.$this->getImportDirectory());
		
		$basename = basename($this->getImportDirectory());

		$xml = $this->getImportDirectory().'/'.$basename.'.xml';
		$qti = $this->getImportDirectory().'/'.preg_replace('/xqsp/', 'qti', $basename).'.xml';
		
		return array($xml,$qti);
	}

	private function getImportDirectoryBase()
	{
		$dir = $this->getImportDirectory();
		$dir = dirname($dir);
		return $dir;
	}
	
	/**
	 * @param array $mapping
	 */
	public function setExternalQplQuestionMapping(array $mapping)
	{
		$this->qpl_qst_mapping = $mapping;
	}
	
	public function getExternalQplQuestionMapping()
	{
		return $this->qpl_qst_mapping;
	}
}
