<?php

include_once("./Services/Repository/classes/class.ilRepositoryObjectPlugin.php");

/**
 * Class ilQuestionSetPoolPlugin
 *
 * Date: 23.09.13
 * Time: 10:33
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilQuestionSetPoolPlugin extends ilRepositoryObjectPlugin
{
	/**
	 * @var self|null
	 */
	private static $instance;

	/**
	 * @var string
	 */
	private static $PLUGIN_CTYPE   = 'Services';

	/**
	 * @var string
	 */
	private static $PLUGIN_CNAME   = 'Repository';

	/**
	 * @var string
	 */
	private static $PLUGIN_SLOT_ID = "robj";

	/**
	 * @var string
	 */
	private static $PLUGIN_ID = "xqsp";

	/**
	 * @var string
	 */
	private static $PLUGIN_NAME = "QuestionSetPool";

	/**
	 * Get the ID of the of the Plugin
	 *
	 * @return string
	 */
	public static function getPluginId()
	{
		return self::$PLUGIN_ID;
	}

	/**
	 * Get Plugin Name. Must be same as in class name il<Name>Plugin
	 * and must correspond to plugins subdirectory name.
	 * Must be overwritten in plugin class of plugin
	 * (and should be made final)
	 * @return    string    Plugin Name
	 */
	public function getPluginName()
	{
		return self::$PLUGIN_NAME;
	}

	public static function getName()
	{
		return self::$PLUGIN_NAME;
	}

	/**
	 * @return self|ilPlugin|ilRepositoryObjectPlugin
	 */
	public static function getInstance()
	{
		if(null !== self::$instance)
		{
			return self::$instance;
		}

		return (self::$instance = ilPluginAdmin::getPluginObject(
			self::$PLUGIN_CTYPE,
			self::$PLUGIN_CNAME,
			self::$PLUGIN_SLOT_ID,
			self::$PLUGIN_NAME
		));
	}

	/**
	 * @return string
	 */
	public function getAdditionalDataTable()
	{
		return "rep_robj_xqsp_data";
	}

	/**
	 * @return string
	 */
	public function getQuestionSetTable()
	{
		return "rep_robj_xqsp_qs";
	}

	/**
	 * @param string $a_type
	 * @param string $a_size
	 * @return string
	 */
	public static function _getIcon($a_type, $a_size)
	{
		return ilPlugin::_getImagePath(
			IL_COMP_SERVICE, 'Repository', 'robj',
			ilPlugin::lookupNameForId(IL_COMP_SERVICE, 'Repository', 'robj', $a_type),
			'icon_' . $a_type . '.svg'
		);
	}

	protected function uninstallCustom()
	{
		/**
		 * @var $ilDB ilDB
		 */
		global $ilDB;

		if($ilDB->tableExists('rep_robj_xqsp_data'))
		{
			$ilDB->dropTable('rep_robj_xqsp_data');
		}
		
		if($ilDB->tableExists('rep_robj_xqsp_jmp_con'))
		{
			$ilDB->dropTable('rep_robj_xqsp_jmp_con');
		}
		if($ilDB->sequenceExists('rep_robj_xqsp_jmp_con'))
		{
			$ilDB->dropSequence('rep_robj_xqsp_jmp_con');
		}
		
		if($ilDB->tableExists('rep_robj_xqsp_node'))
		{
			$ilDB->dropTable('rep_robj_xqsp_node');
		}
		if($ilDB->sequenceExists('rep_robj_xqsp_node'))
		{
			$ilDB->dropSequence('rep_robj_xqsp_node');
		}
		
		if($ilDB->tableExists('rep_robj_xqsp_path'))
		{
			$ilDB->dropTable('rep_robj_xqsp_path');
		}
		if($ilDB->sequenceExists('rep_robj_xqsp_path'))
		{
			$ilDB->dropSequence('rep_robj_xqsp_path');
		}
		
		if($ilDB->tableExists('rep_robj_xqsp_qs'))
		{
			$ilDB->dropTable('rep_robj_xqsp_qs');
		}
		if($ilDB->sequenceExists('rep_robj_xqsp_qs'))
		{
			$ilDB->dropSequence('rep_robj_xqsp_qs');
		}
		
		if($ilDB->tableExists('rep_robj_xqsp_qs_qst'))
		{
			$ilDB->dropTable('rep_robj_xqsp_qs_qst');
		}
	}
}
